
:- ensure_loaded( library(lists) ). % memberchk/2.

:- requires( message_report/3 ).
:- requires( term_template/2 ).
:- requires( select_first/3 ).

%% options( +Required, +Opts ).
%% options( +Required, +Opts, -RemOpts ).
%
% This should more naturally be option/2 but as this might cause confusion with
% Swi's own option library, we named differently.
% Required can be a single or list of terms. 
% RemOpts is Opts after all functor/shape matching mentions of Required
% have been removed.
%
% Opts can also be non list as it is passed through en_list/2.
% 
% The predicate fails silently if the first Required option with equivalent "shape"
% in Opts fails to pattern match (unlike classic memberchk/2).
%
%==
% ?- options( x(y), [x(x)] ).
% false.
% 
% ?- options( x(y), [x(x),x(y)] ).
% false.
%
% ?- options( x(X), [a(b),c(d),x(y),x(x)] ).
% X=y.
%
% ?- options( [a(X),c(b)], [a(b),x(x),b(c)] ).
% ERROR: Required option: c(b), not present in options: [a(b),x(x)]
% % Execution Aborted
% 
% ?- options( x(X), [a(b),c(d),x(y)], Rem ).
% X=y.
% Rem = [a(b), c(d)].
%==
%
% @author nicos angelopoulos
% @version  0.2 2015/01/16
%
options( Needed, OptS ) :-
	en_list( Needed, AllNeeds ),
	en_list( OptS, Opts ),
	maplist( options_option_required(Opts), AllNeeds ).

options_option_required( Opts, Needed ) :-
	term_template( Needed, Termlate ), % we can use select_first/3 here
	memberchk( Termlate, Opts ),
	!,
	Termlate = Needed.
options_option_required( Opts, Needed ) :-
	Mess = 'Required option: ~w, not present in options: ~w',
	message_report( Mess, [Needed,Opts], error ),
	abort.

options( Needed, Opts, Rem ) :-
	en_list( Needed, AllNeeds ),
	options_select( AllNeeds, Opts, Rem ).

options_select( [], Opts, Opts ).
options_select( [Req|Reqs], Opts, Rem ) :-
	select_first( Opts, Req, Rest ),
	options_select( Reqs, Rest, Rem ).
